@@ -2,7 +2,7 @@
  *            qof-sqlite.c
  *
  *  Sun Jan 15 12:52:46 2006
- *  Copyright  2006-2007  Neil Williams
+ *  Copyright  2006-2008  Neil Williams
  *  linux@codehelp.co.uk
  ****************************************************************/
 /*
@@ -30,13 +30,16 @@
 #include <glib.h>
 #include <libintl.h>
 #include "qof.h"
+#include "qofsql-p.h"
+#include "qof-sqlite.h"
+#include "kvputil-p.h"
 
 #define _(String) dgettext (GETTEXT_PACKAGE, String)
 #define ACCESS_METHOD "sqlite"
 
 /** @file  qof-sqlite.c
 	@brief Public interface of qof-backend-sqlite
-	@author Copyright 2006-2007 Neil Williams <linux@codehelp.co.uk>
+	@author Copyright 2006-2008 Neil Williams <linux@codehelp.co.uk>
 */
 
 /** Indicates an item with high priority.  */
@@ -48,6 +51,7 @@
 /** Indicate an error to sqlite */
 #define QSQL_ERROR          -1
 /** One KVP table per file for all instances.  */
+#undef QSQL_KVP_TABLE
 #define QSQL_KVP_TABLE "sqlite_kvp"
 
 #define END_DB_VERSION " dbversion int );"
@@ -55,24 +59,6 @@
 static QofLogModule log_module = QOF_MOD_SQLITE;
 static gboolean loading = FALSE;
 
-typedef enum
-{
-	/** no operation defined. init value. */
-	SQL_NONE = 0,
-	/** Create a new database */
-	SQL_CREATE,
-	/** Load all data from existing database. */
-	SQL_LOAD,
-	/** Write / sync all data to the database. */
-	SQL_WRITE,
-	/** Run a single INSERT statement. */
-	SQL_INSERT,
-	/** Run a single DELETE statement. */
-	SQL_DELETE,
-	/** Run a single UPDATE statement. */
-	SQL_UPDATE
-} QsqlStatementType;
-
 /** \brief Main context for sqlite backend
 
 Contains data elements that persist for longer
@@ -95,7 +81,7 @@ typedef struct
 	/* hashtable relating the GUID to the kvp_id */
 	GHashTable *kvp_id;
 	/* highest kvp_id in the table */
-	glong index;
+	gulong index;
 	QofBook *book;
 	QofErrorId err_delete, err_insert, err_update, err_create;
 } QSQLiteBackend;
@@ -126,100 +112,8 @@ struct QsqlBuilder
 	const QofParam *dirty;
 };
 
-static inline gchar *
-add_to_sql (gchar * sql_str, const gchar * add)
-{
-	gchar *old;
-	old = g_strdup (sql_str);
-	g_free (sql_str);
-	sql_str = g_strconcat (old, add, NULL);
-	g_free (old);
-	return sql_str;
-}
-
-/** \brief Map a KvpValue to a QofIdType
-
- \todo reconcile the duplication with the QSF version
-*/
-static QofIdTypeConst
-kvp_value_to_qof_type_helper (KvpValueType n)
-{
-	switch (n)
-	{
-	case KVP_TYPE_GINT64:
-		{
-			return QOF_TYPE_INT64;
-			break;
-		}
-	case KVP_TYPE_DOUBLE:
-		{
-			return QOF_TYPE_DOUBLE;
-			break;
-		}
-	case KVP_TYPE_NUMERIC:
-		{
-			return QOF_TYPE_NUMERIC;
-			break;
-		}
-	case KVP_TYPE_STRING:
-		{
-			return QOF_TYPE_STRING;
-			break;
-		}
-	case KVP_TYPE_GUID:
-		{
-			return QOF_TYPE_GUID;
-			break;
-		}
-#ifndef QOF_DISABLE_DEPRECATED
-	case KVP_TYPE_TIMESPEC:
-		{
-			return QOF_TYPE_DATE;
-			break;
-		}
-#endif
-	case KVP_TYPE_BOOLEAN:
-		{
-			return QOF_TYPE_BOOLEAN;
-			break;
-		}
-	case KVP_TYPE_TIME:
-		{
-			return QOF_TYPE_TIME;
-			break;
-		}
-	default:
-		{
-			return NULL;
-		}
-	}
-}
-
-/** \todo reconcile the duplication with the QSF version */
-static KvpValueType
-sql_to_kvp_helper (const gchar * type_string)
-{
-	if (0 == safe_strcmp (QOF_TYPE_INT64, type_string))
-		return KVP_TYPE_GINT64;
-	if (0 == safe_strcmp (QOF_TYPE_DOUBLE, type_string))
-		return KVP_TYPE_DOUBLE;
-	if (0 == safe_strcmp (QOF_TYPE_NUMERIC, type_string))
-		return KVP_TYPE_NUMERIC;
-	if (0 == safe_strcmp (QOF_TYPE_STRING, type_string))
-		return KVP_TYPE_STRING;
-	if (0 == safe_strcmp (QOF_TYPE_GUID, type_string))
-		return KVP_TYPE_GUID;
-#ifndef QOF_DISABLE_DEPRECATED
-	if (0 == safe_strcmp (QOF_TYPE_DATE, type_string))
-		return KVP_TYPE_TIMESPEC;
-#endif
-	if (0 == safe_strcmp (QOF_TYPE_TIME, type_string))
-		return KVP_TYPE_TIME;
-	return 0;
-}
-
-/** \todo reconcile the duplication with the QSF version */
-KvpValue *
+/** \todo reconcile the duplication with the QSF (and GDA) version */
+static KvpValue *
 string_to_kvp_value (const gchar * content, KvpValueType type)
 {
 	gchar *tail;
@@ -227,11 +121,6 @@ string_to_kvp_value (const gchar * content, KvpValueType type)
 	gdouble cm_double;
 	QofNumeric cm_numeric;
 	GUID *cm_guid;
-#ifndef QOF_DISABLE_DEPRECATED
-	struct tm kvp_time;
-	time_t kvp_time_t;
-	Timespec cm_date;
-#endif
 
 	switch (type)
 	{
@@ -289,16 +178,6 @@ string_to_kvp_value (const gchar * content, KvpValueType type)
 			else
 				PERR (" failed to parse date");
 		}
-#ifndef QOF_DISABLE_DEPRECATED
-	case KVP_TYPE_TIMESPEC:
-		{
-			strptime (content, QOF_UTC_DATE_FORMAT, &kvp_time);
-			kvp_time_t = mktime (&kvp_time);
-			timespecFromTime_t (&cm_date, kvp_time_t);
-			return kvp_value_new_timespec (cm_date);
-			break;
-		}
-#endif
 	case KVP_TYPE_BOOLEAN:
 		{
 			gboolean val;
@@ -312,7 +191,7 @@ string_to_kvp_value (const gchar * content, KvpValueType type)
 }
 
 /** returns the VALUES for INSERT in pre-defined order */
-static void
+static G_GNUC_UNUSED void
 kvpvalue_to_sql (const gchar * key, KvpValue * val, gpointer builder)
 {
 	QSQLiteBackend *qsql_be;
@@ -335,9 +214,6 @@ kvpvalue_to_sql (const gchar * key, KvpValue * val, gpointer builder)
 	case KVP_TYPE_GUID:
 	case KVP_TYPE_TIME:
 	case KVP_TYPE_BOOLEAN:
-#ifndef QOF_DISABLE_DEPRECATED
-	case KVP_TYPE_TIMESPEC:
-#endif
 		{
 			/* ("kvp_id int primary key not null", "guid char(32)", "path mediumtext",
 			   "type mediumtext", "value text", */
@@ -345,7 +221,7 @@ kvpvalue_to_sql (const gchar * key, KvpValue * val, gpointer builder)
 			qb->sql_str =
 				g_strdup_printf (" kvp key=%s val=%s type=%s", key,
 				kvp_value_to_bare_string (val),
-				kvp_value_to_qof_type_helper (n));
+				kvp_value_type_to_qof_id (n));
 			DEBUG (" %s", qb->sql_str);
 			qb->has_slots = TRUE;
 			break;
@@ -365,132 +241,6 @@ kvpvalue_to_sql (const gchar * key, KvpValue * val, gpointer builder)
 	LEAVE (" %s", qb->sql_str);
 }
 
-static gchar *
-string_param_to_sql (QofParam * param)
-{
-	/* Handle the entity GUID. Ensure that reference GUIDs
-	   must not also try to be primary keys and can be NULL. */
-	if ((0 == safe_strcmp (param->param_type, QOF_TYPE_GUID)) &&
-		(0 == safe_strcmp (param->param_name, QOF_PARAM_GUID)))
-		return g_strdup_printf (" %s char(32) primary key not null",
-			param->param_name);
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_GUID))
-		return g_strdup_printf (" %s char(32)", param->param_name);
-	/* avoid creating database fields for calculated values */
-	if (!param->param_setfcn)
-		return NULL;
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_STRING))
-		return g_strdup_printf (" %s mediumtext", param->param_name);
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_BOOLEAN))
-		return g_strdup_printf (" %s int", param->param_name);
-	if ((0 == safe_strcmp (param->param_type, QOF_TYPE_NUMERIC))
-		|| (0 == safe_strcmp (param->param_type, QOF_TYPE_DOUBLE))
-		|| (0 == safe_strcmp (param->param_type, QOF_TYPE_DEBCRED)))
-	{
-		return g_strdup_printf (" %s text", param->param_name);
-	}
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_INT32))
-		return g_strdup_printf (" %s int", param->param_name);
-#ifndef QOF_DISABLE_DEPRECATED
-	if ((0 == safe_strcmp (param->param_type, QOF_TYPE_DATE)) ||
-		(0 == safe_strcmp (param->param_type, QOF_TYPE_TIME)))
-#else
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_TIME))
-#endif
-		return g_strdup_printf (" %s datetime", param->param_name);
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_CHAR))
-		return g_strdup_printf (" %s char(1)", param->param_name);
-	/* kvp data is stored separately - actually this is really
-	   a no-op because entities do not need a param_setfcn for kvp data. */
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_KVP))
-		return g_strdup ("");
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_COLLECT))
-		return g_strdup_printf (" %s char(32)", param->param_name);
-	/* catch references */
-	return g_strdup_printf (" %s char(32)", param->param_name);
-}
-
-/** \brief list just the parameter names
-
- \note Must match the number and order of the
-list of parameter values from ::create_each_param
-*/
-static void
-create_param_list (QofParam * param, gpointer builder)
-{
-	struct QsqlBuilder *qb;
-	qb = (struct QsqlBuilder *) builder;
-
-	/* avoid creating database fields for calculated values */
-	if (!param->param_setfcn)
-		return;
-	/* avoid setting KVP even if a param_setfcn has been set
-	   because a QofSetterFunc for KVP is quite pointless. */
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_KVP))
-	{
-		PINFO (" kvp support tag");
-		return;
-	}
-	if (!g_str_has_suffix (qb->sql_str, "("))
-	{
-		gchar *add;
-		add = g_strconcat (", ", param->param_name, NULL);
-		qb->sql_str = add_to_sql (qb->sql_str, add);
-		g_free (add);
-	}
-	else
-		qb->sql_str = add_to_sql (qb->sql_str, param->param_name);
-}
-
-/** create the sql for each parameter */
-static void
-create_each_param (QofParam * param, gpointer builder)
-{
-	gchar *value;
-	struct QsqlBuilder *qb;
-	qb = (struct QsqlBuilder *) builder;
-	GList *references;
-
-	/* avoid creating database fields for calculated values */
-	if (!param->param_setfcn)
-		return;
-	/* avoid setting KVP even if a param_setfcn has been set
-	   because a QofSetterFunc for KVP is quite pointless. */
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_KVP))
-		return;
-	references = qof_class_get_referenceList (qb->ent->e_type);
-	if (g_list_find (references, param))
-	{
-		/** \bug will need to use QofEntityReference here
-		if partial books are actually to be supported. */
-		QofEntity *e;
-		e = param->param_getfcn (qb->ent, param);
-		value = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
-		guid_to_string_buff (qof_entity_get_guid (e), value);
-		PINFO (" ref=%p GUID=%s", e, value);
-	}
-	else
-		value = qof_util_param_to_string (qb->ent, param);
-	if (value)
-		g_strescape (value, NULL);
-	if (!value)
-		value = g_strdup ("");
-	if (!g_str_has_suffix (qb->sql_str, "("))
-	{
-		gchar *val;
-		val = g_strconcat (", \"", value, "\"", NULL);
-		qb->sql_str = add_to_sql (qb->sql_str, val);
-		g_free (val);
-	}
-	else
-	{
-		gchar *val;
-		val = g_strconcat ("\"", value, "\"", NULL);
-		qb->sql_str = add_to_sql (qb->sql_str, val);
-		g_free (val);
-	}
-}
-
 /** \brief use the new-style event handlers for insert and update
 insert runs after QOF_EVENT_CREATE
 delete runs before QOF_EVENT_DESTROY
@@ -520,8 +270,7 @@ delete_event (QofEntity * ent, QofEventId event_type,
 				((QofInstance *) ent)->do_free);
 			gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
 			guid_to_string_buff (qof_entity_get_guid (ent), gstr);
-			sql_str = g_strconcat ("DELETE from ", ent->e_type, " WHERE ",
-				QOF_TYPE_GUID, "='", gstr, "';", NULL);
+			sql_str = qof_sql_entity_delete (ent);
 			DEBUG (" sql_str=%s", sql_str);
 			if (sqlite_exec (qsql_be->sqliteh, sql_str,
 					NULL, qsql_be, &qsql_be->err) != SQLITE_OK)
@@ -531,8 +280,6 @@ delete_event (QofEntity * ent, QofEventId event_type,
 				LEAVE (" error on delete:%s", qsql_be->err);
 				break;
 			}
-			/** \todo delete records from QSQL_KVP_TABLE with this GUID */
-			/* SELECT kvp_id from QSQL_KVP_TABLE where guid = gstr */
 			LEAVE (" %d", event_type);
 			qsql_be->error = FALSE;
 			g_free (gstr);
@@ -566,24 +313,17 @@ create_event (QofEntity * ent, QofEventId event_type,
 	{
 	case QOF_EVENT_CREATE:
 		{
-			gchar *tmp;
-			ENTER (" create:%s", ent->e_type);
+			ENTER (" insert:%s", ent->e_type);
 			gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
 			guid_to_string_buff (qof_instance_get_guid ((QofInstance *)
 					ent), gstr);
 			DEBUG (" guid=%s", gstr);
 			qb.ent = ent;
-			qb.sql_str =
-				g_strdup_printf ("INSERT into %s (guid ", ent->e_type);
-			qof_class_param_foreach (ent->e_type, create_param_list, &qb);
-			tmp = g_strconcat (") VALUES (\"", gstr, "\" ", NULL);
-			qb.sql_str = add_to_sql (qb.sql_str, tmp);
-			g_free (tmp);
-			qof_class_param_foreach (ent->e_type, create_each_param, &qb);
-			qb.sql_str = add_to_sql (qb.sql_str, ");");
+			qb.sql_str = qof_sql_entity_insert (ent);
+			/** \bug create one func to create_param_list and create_each_param */
 			DEBUG (" sql_str=%s", qb.sql_str);
 			if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
-					NULL, &qb, &qsql_be->err) != SQLITE_OK)
+				NULL, &qb, &qsql_be->err) != SQLITE_OK)
 			{
 				qof_error_set_be (be, qsql_be->err_insert);
 				qsql_be->error = TRUE;
@@ -603,12 +343,9 @@ create_event (QofEntity * ent, QofEventId event_type,
 			if (slots)
 			{
 				/* id, guid, path, type, value */
-				qb.sql_str = g_strconcat ("INSERT into ", QSQL_KVP_TABLE,
-					"  (kvp_id \"", gstr, "\", ", NULL);
-				kvp_frame_for_each_slot (slots, kvpvalue_to_sql, &qb);
-				qb.sql_str = add_to_sql (qb.sql_str, END_DB_VERSION);
+				qb.sql_str = qof_sql_entity_insert (ent);
 				if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
-						NULL, &qb, &qsql_be->err) != SQLITE_OK)
+					NULL, &qb, &qsql_be->err) != SQLITE_OK)
 				{
 					qof_error_set_be (be, qsql_be->err_insert);
 					qsql_be->error = TRUE;
@@ -639,8 +376,6 @@ qsql_modify (QofBackend * be, QofInstance * inst)
 {
 	struct QsqlBuilder qb;
 	QSQLiteBackend *qsql_be;
-	gchar *gstr, *param_str;
-	KvpFrame *slots;
 
 	qsql_be = (QSQLiteBackend *) be;
 	qb.qsql_be = qsql_be;
@@ -654,16 +389,13 @@ qsql_modify (QofBackend * be, QofInstance * inst)
 		return;
 	ENTER (" modified %s param:%s", ((QofEntity *) inst)->e_type,
 		inst->param->param_name);
-	gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
-	guid_to_string_buff (qof_instance_get_guid (inst), gstr);
-	qb.ent = (QofEntity *) inst;
-	param_str = qof_util_param_to_string (qb.ent, inst->param);
-	if (param_str)
-		g_strescape (param_str, NULL);
-	qb.sql_str = g_strconcat ("UPDATE ", qb.ent->e_type, " SET ",
-		inst->param->param_name, " = \"", param_str,
-		"\" WHERE ", QOF_TYPE_GUID, "='", gstr, "';", NULL);
-	DEBUG (" sql_str=%s param_Str=%s", qb.sql_str, param_str);
+	qb.sql_str = qof_sql_entity_update ((QofEntity*)inst);
+	if (!qb.sql_str)
+	{
+		LEAVE (" null string");
+		return;
+	}
+	DEBUG (" sql_str=%s", qb.sql_str);
 	if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
 			NULL, &qb, &qsql_be->err) != SQLITE_OK)
 	{
@@ -675,36 +407,10 @@ qsql_modify (QofBackend * be, QofInstance * inst)
 	{
 		inst->dirty = FALSE;
 		g_free (qb.sql_str);
-		g_free (gstr);
 		qsql_be->error = FALSE;
 		LEAVE (" ");
 		return;
 	}
-	/* modify slot data */
-	slots = qof_instance_get_slots (inst);
-	if (slots)
-	{
-		/* update and delete KVP data */
-		/* id, guid, path, type, value */
-		qb.sql_str = g_strconcat ("UPDATE ", QSQL_KVP_TABLE,
-			" SET  (kvp_id \"", gstr, "\", ", NULL);
-		kvp_frame_for_each_slot (slots, kvpvalue_to_sql, &qb);
-		qb.sql_str = add_to_sql (qb.sql_str, END_DB_VERSION);
-		if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
-				NULL, &qb, &qsql_be->err) != SQLITE_OK)
-		{
-			qof_error_set_be (be, qsql_be->err_insert);
-			qsql_be->error = TRUE;
-			PERR (" error on KVP create_event:%s", qsql_be->err);
-		}
-		else
-		{
-			((QofInstance *) qb.ent)->dirty = FALSE;
-			qsql_be->error = FALSE;
-			g_free (qb.sql_str);
-		}
-	}
-	g_free (gstr);
 	LEAVE (" ");
 }
 
@@ -752,58 +458,6 @@ record_foreach (gpointer builder, gint col_num, gchar ** strings,
 	return SQLITE_OK;
 }
 
-/* used by create/insert */
-static void
-string_param_foreach (QofParam * param, gpointer builder)
-{
-	struct QsqlBuilder *qb;
-	QSQLiteBackend *qsql_be;
-	gchar *p_str, *old;
-
-	qb = (struct QsqlBuilder *) builder;
-	qsql_be = qb->qsql_be;
-	if (0 == safe_strcmp (param->param_type, QOF_TYPE_KVP))
-		return;
-	p_str = string_param_to_sql (param);
-	/* skip empty values (no param_setfcn) */
-	if (!p_str)
-		return;
-	old = g_strconcat (p_str, ",", NULL);
-	qb->sql_str = add_to_sql (qb->sql_str, old);
-	g_free (old);
-	g_free (p_str);
-}
-
-static void
-update_param_foreach (QofParam * param, gpointer builder)
-{
-	struct QsqlBuilder *qb;
-	gchar *value, *add;
-
-	qb = (struct QsqlBuilder *) builder;
-	if (param != qb->dirty)
-		return;
-	/* update table set name=val,name=val where guid=gstr; */
-	value = qof_util_param_to_string (qb->ent, param);
-	if (value)
-		g_strescape (value, NULL);
-	if (!value)
-		value = g_strdup ("");
-	if (g_str_has_suffix (qb->sql_str, " "))
-	{
-		add = g_strconcat (param->param_name, "=\"", value, "\"", NULL);
-		qb->sql_str = add_to_sql (qb->sql_str, add);
-		g_free (add);
-	}
-	else
-	{
-		add =
-			g_strconcat (",", param->param_name, "=\"", value, "\"", NULL);
-		qb->sql_str = add_to_sql (qb->sql_str, add);
-		g_free (add);
-	}
-}
-
 static void
 update_dirty (gpointer value, gpointer builder)
 {
@@ -812,7 +466,7 @@ update_dirty (gpointer value, gpointer builder)
 	struct QsqlBuilder *qb;
 	QSQLiteBackend *qsql_be;
 	QofBackend *be;
-	gchar *gstr, *param_str;
+	gchar *gstr;
 
 	qb = (struct QsqlBuilder *) builder;
 	qsql_be = qb->qsql_be;
@@ -825,11 +479,12 @@ update_dirty (gpointer value, gpointer builder)
 	gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
 	guid_to_string_buff (qof_entity_get_guid (ent), gstr);
 	/* qof_class_param_foreach  */
-	qb->sql_str = g_strdup_printf ("UPDATE %s SET ", ent->e_type);
-	qof_class_param_foreach (ent->e_type, update_param_foreach, qb);
-	param_str = g_strdup_printf ("WHERE %s=\"%s\";", QOF_TYPE_GUID, gstr);
-	qb->sql_str = add_to_sql (qb->sql_str, param_str);
-	g_free (param_str);
+	qb->sql_str = qof_sql_entity_update (ent);
+	if (!qb->sql_str)
+	{
+		LEAVE (" null string");
+		return;
+	}
 	DEBUG (" update=%s", qb->sql_str);
 	if (sqlite_exec (qsql_be->sqliteh, qb->sql_str,
 			NULL, qb, &qsql_be->err) != SQLITE_OK)
@@ -895,7 +550,6 @@ qsql_create (QofBackend * be, QofInstance * inst)
 	QSQLiteBackend *qsql_be;
 	struct QsqlBuilder qb;
 	QofEntity *ent;
-	KvpFrame *slots;
 
 	qsql_be = (QSQLiteBackend *) be;
 	if (!inst)
@@ -924,16 +578,8 @@ qsql_create (QofBackend * be, QofInstance * inst)
 	}
 	if (!qb.exists)
 	{
-		gchar *add;
 		/* create new entity */
-		qb.sql_str =
-			g_strdup_printf ("INSERT into %s (guid ", ent->e_type);
-		qof_class_param_foreach (ent->e_type, create_param_list, &qb);
-		add = g_strconcat (") VALUES (\"", gstr, "\"", NULL);
-		qb.sql_str = add_to_sql (qb.sql_str, add);
-		g_free (add);
-		qof_class_param_foreach (ent->e_type, create_each_param, &qb);
-		qb.sql_str = add_to_sql (qb.sql_str, ");");
+		qb.sql_str = qof_sql_entity_insert (ent);
 		DEBUG (" sql_str= %s", qb.sql_str);
 		if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
 				NULL, qsql_be, &qsql_be->err) != SQLITE_OK)
@@ -942,32 +588,6 @@ qsql_create (QofBackend * be, QofInstance * inst)
 			qsql_be->error = TRUE;
 			PERR (" error creating new entity:%s", qsql_be->err);
 		}
-		/* KVP here */
-		slots = qof_instance_get_slots ((QofInstance *) ent);
-		if (slots)
-		{
-			/* id, guid, path, type, value */
-			qb.sql_str = g_strconcat ("INSERT into ", QSQL_KVP_TABLE,
-				"  (kvp_id, \"", gstr, "\", ", NULL);
-			kvp_frame_for_each_slot (slots, kvpvalue_to_sql, &qb);
-			qb.sql_str = add_to_sql (qb.sql_str, ");");
-		}
-		if (qb.has_slots)
-		{
-			if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
-					NULL, &qb, &qsql_be->err) != SQLITE_OK)
-			{
-				qof_error_set_be (be, qsql_be->err_insert);
-				qsql_be->error = TRUE;
-				PERR (" error on KVP create_event:%s:%s", qsql_be->err,
-					qb.sql_str);
-			}
-			else
-			{
-				((QofInstance *) ent)->dirty = FALSE;
-				qsql_be->error = FALSE;
-			}
-		}
 	}
 	g_free (qb.sql_str);
 	g_free (gstr);
@@ -983,7 +603,6 @@ check_state (QofEntity * ent, gpointer builder)
 	struct QsqlBuilder *qb;
 	QofBackend *be;
 	QofInstance *inst;
-	KvpFrame *slots;
 
 	qb = (struct QsqlBuilder *) builder;
 	qsql_be = qb->qsql_be;
@@ -1016,16 +635,8 @@ check_state (QofEntity * ent, gpointer builder)
 	}
 	if (!qb->exists)
 	{
-		gchar *add;
 		/* create new entity */
-		qb->sql_str =
-			g_strdup_printf ("INSERT into %s (guid ", ent->e_type);
-		qof_class_param_foreach (ent->e_type, create_param_list, &qb);
-		add = g_strconcat (") VALUES (\"", gstr, "\" ", NULL);
-		qb->sql_str = add_to_sql (qb->sql_str, add);
-		g_free (add);
-		qof_class_param_foreach (ent->e_type, create_each_param, &qb);
-		qb->sql_str = add_to_sql (qb->sql_str, ");");
+		qb->sql_str = qof_sql_entity_insert (ent);
 		DEBUG (" sql_str= %s", qb->sql_str);
 		if (sqlite_exec (qsql_be->sqliteh, qb->sql_str,
 				NULL, qb, &qsql_be->err) != SQLITE_OK)
@@ -1035,28 +646,6 @@ check_state (QofEntity * ent, gpointer builder)
 			PERR (" error on check_state create_new:%s", qsql_be->err);
 		}
 		g_free (qb->sql_str);
-		/* create KVP data too */
-		slots = qof_instance_get_slots ((QofInstance *) ent);
-		if (slots)
-		{
-			/* id, guid, path, type, value */
-			qb->sql_str = g_strconcat ("INSERT into ", QSQL_KVP_TABLE,
-				"  (kvp_id \"", gstr, "\", ", NULL);
-			kvp_frame_for_each_slot (slots, kvpvalue_to_sql, &qb);
-			qb->sql_str = add_to_sql (qb->sql_str, END_DB_VERSION);
-			if (sqlite_exec (qsql_be->sqliteh, qb->sql_str,
-					NULL, &qb, &qsql_be->err) != SQLITE_OK)
-			{
-				qof_error_set_be (be, qsql_be->err_insert);
-				qsql_be->error = TRUE;
-				PERR (" error on KVP create_event:%s", qsql_be->err);
-			}
-			else
-			{
-				((QofInstance *) ent)->dirty = FALSE;
-				qsql_be->error = FALSE;
-			}
-		}
 	}
 	/* update instead */
 	g_list_foreach (qb->dirty_list, update_dirty, &qb);
@@ -1081,7 +670,7 @@ build_kvp_table (gpointer builder, gint col_num, gchar ** strings,
 	KvpFrame *frame;
 	KvpValueType type;
 	KvpValue *value;
-	glong max;
+	gulong max;
 	gchar *tail;
 
 	g_return_val_if_fail (builder, QSQL_ERROR);
@@ -1094,7 +683,7 @@ build_kvp_table (gpointer builder, gint col_num, gchar ** strings,
 	/* columnNames = fields strings = values
 	   [0]=kvp_id, [1]=guid, [2]=path, [3]=type, [4]=value
 	   get type from type_string */
-	type = sql_to_kvp_helper (strings[3]);
+	type = qof_id_to_kvp_value_type (strings[3]);
 	if (type == 0)
 	{
 		PERR (" invalid type returned from kvp table");
@@ -1188,11 +777,7 @@ qsql_class_foreach (QofObject * obj, gpointer data)
 	case SQL_CREATE:
 		{
 			/* KVP is handled separately */
-			qb.sql_str =
-				g_strdup_printf ("CREATE TABLE %s (", obj->e_type);
-			qof_class_param_foreach (obj->e_type, string_param_foreach,
-				&qb);
-			qb.sql_str = add_to_sql (qb.sql_str, END_DB_VERSION);
+			qb.sql_str = qof_sql_object_create_table (obj);
 			if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
 					NULL, NULL, &qsql_be->err) != SQLITE_OK)
 			{
@@ -1264,22 +849,6 @@ qsql_backend_createdb (QofBackend * be, QofSession * session)
 		return;
 	}
 	qof_object_foreach_type (qsql_class_foreach, qsql_be);
-	/* create the KVP table here
-	   preset table name, internal_id, guid_as_string, path, type, value
-	 */
-	qb.sql_str =
-		g_strdup_printf ("CREATE TABLE %s (%s, %s, %s, %s, %s, %s",
-		QSQL_KVP_TABLE, "kvp_id int primary key not null",
-		"guid char(32)", "path mediumtext", "type mediumtext",
-		"value text", END_DB_VERSION);
-	PINFO (" sql=%s", qb.sql_str);
-	if (sqlite_exec (qsql_be->sqliteh, qb.sql_str,
-			record_foreach, &qb, &qsql_be->err) != SQLITE_OK)
-	{
-		qsql_be->error = TRUE;
-		PERR (" unable to create kvp table:%s", qsql_be->err);
-	}
-	g_free (qb.sql_str);
 	LEAVE (" ");
 }
 
@@ -1382,7 +951,6 @@ qsqlite_write_db (QofBackend * be, QofBook * book)
 	qsql_be->book = book;
 	/* update each record with current state */
 	qof_object_foreach_type (qsql_class_foreach, qsql_be);
-	/* update KVP */
 }
 
 static gboolean
@@ -1500,8 +1068,9 @@ qof_sqlite_provider_init (void)
 
 	ENTER (" ");
 	bindtextdomain (PACKAGE, LOCALE_DIR);
+	qof_sql_entity_set_kvp_tablename (QSQL_KVP_TABLE);
 	prov = g_new0 (QofBackendProvider, 1);
-	prov->provider_name = "QOF SQLite Backend Version 0.3";
+	prov->provider_name = "QOF SQLite Backend Version 0.4";
 	prov->access_method = ACCESS_METHOD;
 	prov->partial_book_supported = TRUE;
 	prov->backend_new = qsql_backend_new;
