@@ -24,7 +24,11 @@
     @file qofsql.c
     @brief QOF client-side SQL parser - interfaces with libgda.
     @author Copyright (C) 2004 Linas Vepstas <linas@linas.org>
+    @author Copyright 2008 Neil Williams <linux@codehelp.co.uk>
 
+ Intended to parse incoming SQL into QOF queries (SELECT or INSERT)
+ and prepare SQL commands from QOF entities (CREATE, UPDATE, INSERT,
+ DELETE and DROP) for use in SQL-based backends.
 */
 
 #include "config.h"
@@ -39,9 +43,16 @@
 #endif
 #include <time.h>
 #include "qof.h"
+#include "qofsql-p.h"
 #include "qofquery-p.h"
 
 #define _(String) dgettext (GETTEXT_PACKAGE, String)
+/** One KVP table per file for all instances. */
+static gchar * kvp_table_name = NULL;
+
+#define QSQL_KVP_TABLE "sql_kvp"
+
+#define END_DB_VERSION " dbversion int );"
 
 static QofLogModule log_module = QOF_MOD_QUERY;
 
@@ -150,7 +161,7 @@ dequote_string (char *str)
 static QofQuery *
 handle_single_condition (QofSqlQuery * query, sql_condition * cond)
 {
-	char tmpbuff[128];
+	gchar tmpbuff[128];
 	GSList *param_list;
 	GList *guid_list;
 	QofQueryPredData *pred_data;
@@ -258,9 +269,6 @@ handle_single_condition (QofSqlQuery * query, sql_condition * cond)
 			break;
 		case KVP_TYPE_GUID:
 		case KVP_TYPE_TIME :
-#ifndef QOF_DISABLE_DEPRECATED
-		case KVP_TYPE_TIMESPEC:
-#endif
 		case KVP_TYPE_BOOLEAN :
 		case KVP_TYPE_BINARY:
 		case KVP_TYPE_GLIST:
@@ -377,29 +385,6 @@ handle_single_condition (QofSqlQuery * query, sql_condition * cond)
 			qof_query_time_predicate (qop, QOF_DATE_MATCH_NORMAL, 
 			qt);
 	}
-#ifndef QOF_DISABLE_DEPRECATED
-	else if (!strcmp (param_type, QOF_TYPE_DATE))
-	{
-		gint rc;
-		Timespec ts;
-		time_t exact;
-
-		/* Use a timezone independent setting */
-		qof_date_format_set (QOF_DATE_FORMAT_UTC);
-		rc = 0;
-		if (FALSE == qof_scan_date_secs (qvalue_name, &exact))
-		{
-			char *tail;
-			exact = strtoll (qvalue_name, &tail, 0);
-//          PWARN ("unable to parse date: %s", qvalue_name);
-//          return NULL;
-		}
-		ts.tv_sec = exact;
-		ts.tv_nsec = 0;
-		pred_data =
-			qof_query_date_predicate (qop, QOF_DATE_MATCH_NORMAL, ts);
-	}
-#endif
 	else if (!strcmp (param_type, QOF_TYPE_NUMERIC))
 	{
 		QofNumeric ival;
@@ -684,25 +669,6 @@ qof_sql_insertCB (const QofParam * param, const gchar * insert_string,
 			time_setter (ent, qt);
 		}
 	}
-#ifndef QOF_DISABLE_DEPRECATED
-	if (safe_strcmp (param->param_type, QOF_TYPE_DATE) == 0)
-	{
-		void (*date_setter) (QofEntity *, Timespec);
-		Timespec cm_date;
-		struct tm query_time;
-		time_t query_time_t;
-
-		date_setter =
-			(void (*)(QofEntity *, Timespec)) param->param_setfcn;
-		strptime (insert_string, QOF_UTC_DATE_FORMAT, &query_time);
-		query_time_t = mktime (&query_time);
-		timespecFromTime_t (&cm_date, query_time_t);
-		if (date_setter != NULL)
-		{
-			date_setter (ent, cm_date);
-		}
-	}
-#endif
 	if ((safe_strcmp (param->param_type, QOF_TYPE_NUMERIC) == 0) ||
 		(safe_strcmp (param->param_type, QOF_TYPE_DEBCRED) == 0))
 	{
@@ -1098,4 +1064,516 @@ qof_sql_query_rerun (QofSqlQuery * query)
 	return results;
 }
 
+/** simple helper struct for passing between SQL-generating functions */
+typedef struct ent_and_string
+{
+	QofEntity * ent;
+	gchar * str;
+	gchar * kvp_str;
+	gchar * full_kvp_path;
+}eas;
+
+static gulong kvp_id = 0;
+static gboolean kvp_table_exists = FALSE;
+
+static void
+create_sql_from_param_cb (QofParam * param, gpointer user_data)
+{
+	eas * data;
+	gchar * value;
+	GList * references;
+
+	g_return_if_fail (param);
+	data = (eas*) user_data;
+	ENTER ("%s %s", data->ent->e_type, param->param_name);
+	g_return_if_fail (data->ent);
+	if (!data->str)
+		data->str = g_strdup ("");
+	/* avoid creating database fields for calculated values */
+	if (!param->param_setfcn)
+		return;
+	/* avoid setting KVP even if a param_setfcn has been set
+	   because a QofSetterFunc for KVP is quite pointless. */
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_KVP))
+		return;
+	references = qof_class_get_referenceList (data->ent->e_type);
+	if (g_list_find (references, param))
+	{
+		/* \bug will need to use QofEntityReference here
+		if partial books are actually to be supported. */
+		QofEntity *e;
+		e = param->param_getfcn (data->ent, param);
+		value = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
+		guid_to_string_buff (qof_entity_get_guid (e), value);
+		PINFO (" ref=%p GUID=%s", e, value);
+	}
+	else
+		value = qof_util_param_to_string (data->ent, param);
+	if (value)
+		g_strescape (value, NULL);
+	if (!value)
+		value = g_strdup ("");
+	if (!g_str_has_suffix (data->str, "("))
+	{
+		gchar *tmp;
+		tmp = g_strjoin ("", data->str, ", '", value, "'", NULL);
+		g_free (data->str);
+		data->str = tmp;
+	}
+	else
+	{
+		gchar * tmp;
+		tmp = g_strjoin ("", data->str, "'", value, "'", NULL);
+		g_free (data->str);
+		data->str = tmp;
+	}
+	LEAVE ("%s", data->str);
+}
+
+static gchar *
+string_param_to_sql (QofParam * param)
+{
+	/** \note These strings are fairly standard SQL
+	  but SQL means different things to different programs.
+	  Keep these strings as simple and plain as possible.
+	  Hopefully, what works in SQLite0 will work in most
+	  other SQL based programs.
+	  */
+	/* Handle the entity GUID. Ensure that reference GUIDs
+	   must not also try to be primary keys and can be NULL. */
+	if ((0 == safe_strcmp (param->param_type, QOF_TYPE_GUID)) &&
+		(0 == safe_strcmp (param->param_name, QOF_PARAM_GUID)))
+		return g_strdup_printf (" %s char(32) primary key not null",
+			param->param_name);
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_GUID))
+		return g_strdup_printf (" %s char(32)", param->param_name);
+	/* avoid creating database fields for calculated values */
+	if (!param->param_setfcn)
+		return NULL;
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_STRING))
+		return g_strdup_printf (" %s mediumtext", param->param_name);
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_BOOLEAN))
+		return g_strdup_printf (" %s int", param->param_name);
+	if ((0 == safe_strcmp (param->param_type, QOF_TYPE_NUMERIC))
+		|| (0 == safe_strcmp (param->param_type, QOF_TYPE_DOUBLE))
+		|| (0 == safe_strcmp (param->param_type, QOF_TYPE_DEBCRED)))
+	{
+		return g_strdup_printf (" %s text", param->param_name);
+	}
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_INT32))
+		return g_strdup_printf (" %s int", param->param_name);
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_TIME))
+		return g_strdup_printf (" %s datetime", param->param_name);
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_CHAR))
+		return g_strdup_printf (" %s char(1)", param->param_name);
+	/* kvp data is stored separately - actually this is really
+	   a no-op because entities do not need a param_setfcn for kvp data. */
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_KVP))
+		return g_strdup ("");
+/**  \bug This won't work - collect types can vary unpredictably - need
+	a KVP implementation. */
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_COLLECT))
+		return g_strdup ("");
+//		return g_strdup_printf (" %s char(32)", param->param_name);
+	/* catch references */
+	return g_strdup_printf (" %s char(32)", param->param_name);
+}
+
+/* used by create/insert */
+static void
+string_param_foreach (QofParam * param, gpointer user_data)
+{
+	gchar *p_str, * tmp;
+	eas * data;
+
+	PINFO ("string_param_foreach:%s", param->param_name);
+	data = (eas *)user_data;
+	if ((0 == safe_strcmp (param->param_type, QOF_TYPE_KVP)) &&
+		(kvp_table_exists == FALSE))
+	{
+		g_free (data->kvp_str);
+		data->kvp_str = 
+			g_strdup_printf (" CREATE TABLE %s (%s, %s, %s, %s, %s, %s",
+			kvp_table_name, "kvp_id int primary key not null",
+			"guid char(32)", "path mediumtext", "type mediumtext",
+			"value text", END_DB_VERSION);
+		return;
+	}
+	p_str = string_param_to_sql (param);
+	/* skip empty values (no param_setfcn) */
+	if (!p_str)
+		return;
+	if (!data->str)
+		data->str = g_strdup("");
+	tmp = g_strjoin ("", data->str, p_str, ",", NULL);
+	g_free (data->str);
+	data->str = tmp;
+	g_free (p_str);
+}
+
+/** \brief list just the parameter names
+
+ \note Must match the number and order of the
+list of parameter values from ::create_each_param
+*/
+static void
+create_param_list (QofParam * param, gpointer user_data)
+{
+	eas * data = (eas *)user_data;
+	g_return_if_fail (data->str);
+	/* avoid creating database fields for calculated values */
+	if (!param->param_setfcn)
+		return;
+	/* avoid setting KVP even if a param_setfcn has been set
+	   because a QofSetterFunc for KVP is quite pointless. */
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_KVP))
+	{
+		PINFO (" kvp support tag");
+		return;
+	}
+	if (!g_str_has_suffix (data->str, "("))
+	{
+		gchar *add;
+		add = g_strconcat (data->str, ", ", param->param_name, NULL);
+		g_free (data->str);
+		data->str = add;
+	}
+	else
+	{
+		gchar * add;
+		add = g_strjoin ("", data->str, param->param_name, NULL);
+		g_free (data->str);
+		data->str = add;
+	}
+}
+
+/** returns the VALUES for INSERT in pre-defined order */
+static void
+kvpvalue_to_sql_insert (const gchar * key, KvpValue * val, gpointer user_data)
+{
+	eas * data;
+	KvpValueType n;
+	gchar * path;
+
+	path = g_strdup("");
+	ENTER (" ");
+	data = (eas*)user_data;
+	g_return_if_fail (key && val && data);
+	n = kvp_value_get_type (val);
+	switch (n)
+	{
+	case KVP_TYPE_GINT64:
+	case KVP_TYPE_DOUBLE:
+	case KVP_TYPE_NUMERIC:
+	case KVP_TYPE_STRING:
+	case KVP_TYPE_GUID:
+	case KVP_TYPE_TIME:
+	case KVP_TYPE_BOOLEAN:
+		{
+			path = g_strjoin ("/", data->full_kvp_path, key, NULL);
+			data->str =
+				g_strdup_printf ("'%s', '%s', '%s'", key, path,
+				kvp_value_to_bare_string (val));
+			DEBUG (" %s", data->str);
+			break;
+		}
+	case KVP_TYPE_FRAME:
+		{
+			path = g_strjoin ("/", data->full_kvp_path, key, NULL);
+			g_free (data->full_kvp_path);
+			data->full_kvp_path = path;
+			kvp_frame_for_each_slot (kvp_value_get_frame (val),
+				kvpvalue_to_sql_insert, data);
+			break;
+		}
+	default:
+		{
+			PERR (" unsupported value = %d", kvp_value_get_type (val));
+			break;
+		}
+	}
+	LEAVE (" %s", data->str);
+}
+
+/** returns the VALUES for UPDATE in pre-defined order */
+static void
+kvpvalue_to_sql_update (const gchar * key, KvpValue * val, gpointer user_data)
+{
+	eas * data;
+	KvpValueType n;
+	gchar * path;
+
+	ENTER (" key=%s", key);
+	data = (eas*)user_data;
+	g_return_if_fail (key && val && data);
+	n = kvp_value_get_type (val);
+	switch (n)
+	{
+	case KVP_TYPE_GINT64:
+	case KVP_TYPE_DOUBLE:
+	case KVP_TYPE_NUMERIC:
+	case KVP_TYPE_STRING:
+	case KVP_TYPE_GUID:
+	case KVP_TYPE_TIME:
+	case KVP_TYPE_BOOLEAN:
+		{
+			path = g_strjoin ("/", data->full_kvp_path, key, NULL);
+			data->str =
+				g_strdup_printf ("type='%s', value='%s' WHERE path='%s' and ", 
+					key, kvp_value_to_bare_string (val), path);
+			DEBUG (" %s", data->str);
+			break;
+		}
+	case KVP_TYPE_FRAME:
+		{
+			path = g_strjoin ("/", data->full_kvp_path, key, NULL);
+			g_free (data->full_kvp_path);
+			data->full_kvp_path = path;
+			kvp_frame_for_each_slot (kvp_value_get_frame (val),
+				kvpvalue_to_sql_update, data);
+			break;
+		}
+	default:
+		{
+			PERR (" unsupported value = %d", kvp_value_get_type (val));
+			break;
+		}
+	}
+	LEAVE (" %s", data->str);
+}
+
+gchar *
+qof_sql_object_create_table (QofObject * obj)
+{
+	gchar * sql_str, * start;
+	eas data;
+
+	if (!kvp_table_name)
+		kvp_table_name = g_strdup(QSQL_KVP_TABLE);
+	ENTER ("create table for %s", obj->e_type);
+	start = g_strdup_printf ("CREATE TABLE %s (", obj->e_type);
+	data.ent = NULL;
+	data.str = g_strdup("");
+	data.kvp_str = g_strdup("");
+	qof_class_param_foreach (obj->e_type, string_param_foreach, &data);
+	sql_str = g_strjoin ("", start, data.str, END_DB_VERSION, data.kvp_str, NULL);
+	g_free (start);
+	g_free (data.kvp_str);
+	g_free (data.str);
+	LEAVE ("sql_str=%s", sql_str);
+	return sql_str;
+}
+
+gchar *
+qof_sql_entity_create_table (QofEntity * ent)
+{
+	gchar * sql_str, * start;
+	eas data;
+
+	g_return_val_if_fail (ent, NULL);
+	if (!kvp_table_name)
+		kvp_table_name = g_strdup(QSQL_KVP_TABLE);
+	ENTER ("create table for %s", ent->e_type);
+	start = g_strdup_printf ("CREATE TABLE %s (", ent->e_type);
+	data.ent = ent;
+	data.str = g_strdup("");
+	data.kvp_str = g_strdup("");
+	qof_class_param_foreach (ent->e_type, string_param_foreach, &data);
+	sql_str = g_strjoin ("", start, data.str, END_DB_VERSION, data.kvp_str, NULL);
+	g_free (start);
+	LEAVE ("sql_str=%s", sql_str);
+	return sql_str;
+}
+
+gchar *
+qof_sql_entity_insert (QofEntity * ent)
+{
+	KvpFrame * slots;
+	eas data;
+	gchar * command, * fields, * values, *id, *gstr, *sql_str, *kvp;
+
+	data.ent = ent;
+	data.str = g_strdup("");
+	if (!kvp_table_name)
+		kvp_table_name = g_strdup(QSQL_KVP_TABLE);
+	ENTER (" insert a single '%s'", ent->e_type);
+	gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
+	guid_to_string_buff (qof_instance_get_guid ((QofInstance *) ent), gstr);
+	DEBUG (" guid=%s", gstr);
+	command = g_strdup_printf ("INSERT into %s (guid ", ent->e_type);
+	// store param list in fields
+	qof_class_param_foreach (ent->e_type, create_param_list, &data);
+	fields = g_strdup(data.str);
+	// store param values in values
+	g_free (data.str);
+	data.str = g_strdup("");
+	data.full_kvp_path = g_strdup("");
+	qof_class_param_foreach (ent->e_type, create_sql_from_param_cb, &data);
+	values = data.str;
+	/* handle KVP */
+	kvp = g_strdup("");
+	slots = qof_instance_get_slots ((QofInstance *) ent);
+	if (!kvp_frame_is_empty(slots))
+	{
+		id = g_strdup_printf ("%lu", kvp_id);
+		g_free (kvp);
+		kvp_frame_for_each_slot (slots, kvpvalue_to_sql_insert, &data);
+		kvp = g_strconcat (" INSERT into ", kvp_table_name,
+			"  (kvp_id, guid, type, path, value) VALUES ('", id, "', '", 
+			gstr, "', ", data.str, ");", NULL);
+		/* increment the index value of the KVP table */
+		kvp_id++;
+		g_free (data.str);
+	}
+	sql_str = g_strjoin ("", command, fields, ") VALUES ('", gstr, "' ", 
+		values, ");", kvp, NULL);
+	g_free (command);
+	g_free (fields);
+	g_free (gstr);
+	g_free (values);
+	g_free (data.full_kvp_path);
+	LEAVE ("sql_str=%s", sql_str);
+	return sql_str;
+}
+
+static void
+collect_kvp (QofEntity * ent, gpointer user_data)
+{
+	KvpFrame * slots;
+	KvpValue * collguid;
+	gchar * gstr, * name, * key;
+	const QofParam * param;
+
+	name = (gchar *) user_data;
+	param = qof_class_get_parameter (ent->e_type, name);
+	key = g_strconcat ("collection/", name, "/guid/", NULL);
+	gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
+	guid_to_string_buff (qof_instance_get_guid ((QofInstance*)ent), gstr);
+	collguid = kvp_value_new_string(gstr);
+	slots = qof_instance_get_slots((QofInstance*)ent);
+	kvp_frame_set_value (slots, key, collguid);
+	g_free (key);
+	g_free (gstr);
+}
+
+gchar *
+qof_sql_entity_update (QofEntity * ent)
+{
+	gchar *gstr, * sql_str, * param_str;
+	QofInstance * inst;
+	const QofParam * param;
+	inst = (QofInstance*)ent;
+
+	if (!inst->param)
+		return NULL;
+	ENTER (" modified %s param:%s", ent->e_type, inst->param->param_name);
+	gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
+	guid_to_string_buff (qof_instance_get_guid (inst), gstr);
+	param = inst->param;
+	if (0 == safe_strcmp (param->param_type, QOF_TYPE_COLLECT))
+	{
+		gchar * name;
+		QofCollection * coll;
+
+		coll = param->param_getfcn (ent, param);
+		name = g_strdup (param->param_name);
+		qof_collection_foreach (coll, collect_kvp, name);
+		g_free (name);
+		return NULL;
+	}
+	param_str = qof_util_param_to_string (ent, inst->param);
+	if (param_str)
+		g_strescape (param_str, NULL);
+	sql_str = g_strconcat ("UPDATE ", ent->e_type, " SET ",
+		inst->param->param_name, " = '", param_str,
+		"' WHERE ", QOF_TYPE_GUID, "='", gstr, "';", NULL);
+	LEAVE ("sql_str=%s", sql_str);
+	return sql_str;
+}
+
+gchar *
+qof_sql_entity_update_kvp (QofEntity * ent)
+{
+	eas data;
+	gchar *gstr, * sql_str;
+	QofInstance * inst;
+	gchar * start;
+	KvpFrame * slots;
+	inst = (QofInstance*)ent;
+
+	if (!inst->param)
+		return NULL;
+	sql_str = NULL;
+	if (kvp_frame_is_empty (qof_instance_get_slots ((QofInstance*)ent)))
+		return NULL;
+
+	ENTER (" modified %s param:%s", ent->e_type, inst->param->param_name);
+	gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
+	guid_to_string_buff (qof_instance_get_guid (inst), gstr);
+	data.str = g_strdup("");
+	data.full_kvp_path = g_strdup("");
+	slots = qof_instance_get_slots ((QofInstance*)ent);
+	start = g_strjoin ("", "UPDATE ", kvp_table_name, " SET ", NULL);
+	/** \note the WHERE condition tests the path and the GUID
+	as each entity (one GUID) can have more than one KVP. */
+	kvp_frame_for_each_slot (slots, kvpvalue_to_sql_update, &data);
+	sql_str = g_strjoin ("", start, data.str, " guid='", gstr, "';", NULL);
+	g_free (start);
+	g_free (data.full_kvp_path);
+	g_free (data.str);
+	LEAVE ("sql_str=%s", sql_str);
+	return sql_str;
+}
+
+gchar *
+qof_sql_entity_update_list (QofEntity * ent, GList **params)
+{
+	return NULL;
+}
+
+gchar *
+qof_sql_entity_delete (QofEntity * ent)
+{
+	gchar * gstr, * sql_str;
+	ENTER (" %s", ent->e_type);
+	gstr = g_strnfill (GUID_ENCODING_LENGTH + 1, ' ');
+	guid_to_string_buff (qof_entity_get_guid (ent), gstr);
+	sql_str = g_strconcat ("DELETE from ", ent->e_type, " WHERE ",
+		QOF_TYPE_GUID, "='", gstr, "';", "DELETE from ", kvp_table_name, 
+		" WHERE kvp_id ", "='", gstr, "';", NULL);
+	g_free (gstr);
+	return sql_str;
+}
+
+gchar *
+qof_sql_entity_drop_table (QofEntity * ent)
+{
+	gchar * sql_str;
+	ENTER (" drop table for '%s'", ent->e_type);
+	sql_str = g_strdup_printf ("DROP TABLE %s;", ent->e_type);
+	LEAVE ("sql_str=%s", sql_str);
+	return sql_str;
+}
+
+void qof_sql_entity_set_kvp_tablename (const gchar * name)
+{
+	g_return_if_fail (name);
+	kvp_table_name = g_strdup(name);
+}
+
+void qof_sql_entity_set_kvp_id (gulong id)
+{
+	kvp_id = id;
+}
+
+gulong qof_sql_entity_get_kvp_id (void)
+{
+	return kvp_id;
+}
+
+void qof_sql_entity_set_kvp_exists (gboolean exist)
+{
+	kvp_table_exists = exist;
+}
+
 /* ========================== END OF FILE =================== */
